@@ -77,6 +77,8 @@ static tHttpUrl baseUrl, proxyUrl;
 static mstring altPath;
 bool g_bGoodServer=true;
 
+cmstring sDefPortHTTP("3142"), sDefPortHTTPS("80");
+
 struct tDlDesc
 {
 	cmstring m_path;
@@ -881,8 +883,9 @@ int main(int argc, char *argv[])
    
    if(argv[2] && proxyUrl.SetHttpUrl(argv[2]))
    {
-	   if(proxyUrl.sPort.empty())
+	   /*if(proxyUrl.GetPort().empty())
 		   proxyUrl.sPort="3142";
+		   */
    }
    else
    {
