@@ -45,7 +45,7 @@ bool SetOption(const mstring &line, bool bQuiet=false, NoCaseStringMap *pDupeChe
 void ReadConfigDirectory(const char*, bool bTestMode);
 
 //! Prepare various things resulting from variable combinations, etc.
-void PostProcConfig();
+void PostProcConfig(bool bDumpConfig);
 
 // TODO: document me
 // throw away the rewritten part of the path, foo/debian/bla.deb -> bla.deb,
@@ -67,7 +67,8 @@ struct tRepoData
 	IHookHandler *m_pHooks;
 	tStrVec m_keyfiles;
 	tHttpUrl m_deltasrc;
-	tRepoData() : m_pHooks(NULL) {};
+	tHttpUrl *m_pProxy;
+	tRepoData() : m_pHooks(NULL), m_pProxy(NULL) {};
 	virtual ~tRepoData();
 };
 
