@@ -70,11 +70,7 @@ namespace aclog
       };
 
       mstring GetStatReport();
-      
-      struct errnoFmter : public mstring
-      {
-    	  errnoFmter(const char *prefix=NULL);
-      };
+
 }
 
 //#define TIMEFORMAT "%a %d/%m"
