@@ -28,15 +28,12 @@ public:
 	virtual void Run(const mstring &cmd);
 
 protected:
-	bool CheckAbortCondition();
+	bool CheckStopSignal();
 
 	// to be implemented by subclasses
 	virtual void Action(const mstring &) =0;
 
-	using tWUIPage::SendChunk;
-
-	/// pass it to the base class but store a copy to some log file
-	virtual void SendChunk(const char *data, size_t size);
+	void AfterSendChunk(const char *data, size_t size);
 
 	void DumpLog(time_t id);
 
@@ -62,8 +59,9 @@ private:
 		tProgressTracker() : nEndSize(-1), id(0) {};
 		void SetEnd(off_t);
 	};
+	typedef SMARTPTR_SPACE::shared_ptr<tProgressTracker> tProgTrackPtr;
 	static SMARTPTR_SPACE::weak_ptr<tProgressTracker> g_pTracker;
-	SMARTPTR_SPACE::shared_ptr<tProgressTracker> m_pTracker;
+	tProgTrackPtr m_pTracker;
 };
 
 struct tRemoteFileInfo;
