@@ -32,7 +32,7 @@ public:
 	static SPtr CachedResolve(const mstring & sHostname, const mstring &sPort,
 			mstring &sErrorMsgBuf);
 
-	static time_t ExpireCache();
+	static time_t BackgroundCleanup();
 
 protected:
 	struct addrinfo * m_resolvedInfo; // getaddrinfo excrements, to cleanup
