@@ -23,6 +23,7 @@
 #define LOGSTART(x)
 #define LOGSTART2(x,y)
 #define LOGSTART2s(x,y)
+#define DBGQLOG(x)
 
 #else
 
@@ -37,6 +38,7 @@
 #define ldbg(x) LOG(x)
 
 #define dbgline ldbg("mark")
+#define DBGQLOG(x) {aclog::err(tSS()<< x);}
 
 #endif
 
