@@ -43,7 +43,7 @@ class dlcon : public lockable
     	dlcon & operator=(const dlcon&);
     	dlcon(const dlcon&);
     	
-    	//friend class tDlJob;
+    	friend class tDlJob;
     	
     	tDljQueue m_qNewjobs;
 
@@ -79,7 +79,9 @@ class dlcon : public lockable
     	int m_nTempPipelineDisable;
 
 
-    	bool m_proxied;
+    	// the default behavior or using or not using the proxy. Will be set
+    	// if access proxies shall no longer be used.
+    	bool m_bProxyTot;
 };
 
 #endif
