@@ -3,7 +3,7 @@
 
 #include "cacheman.h"
 
-class expiration : public tCacheMan
+class expiration : public tCacheMan, ifileprocessor
 {
 public:
 
