@@ -24,11 +24,7 @@
 #define O_BINARY 0 // ignore on Unix
 #endif
 
-#ifdef HAVE_LINUX_FALLOCATE
 int falloc_helper(int fd, off_t start, off_t len);
-#else
-#define falloc_helper(fd,start,len)
-#endif
 
 ssize_t sendfile_generic(int out_fd, int in_fd, off_t *offset, size_t count);
 
@@ -55,6 +51,7 @@ bool LinkOrCopy(const mstring &from, const mstring &to);
 
 
 void set_nb(int fd);
+void set_block(int fd);
 
 #define forceclose(fd) { while(0 != ::close(fd)) { if(errno != EINTR) break; }; fd=-1; }
 #define checkforceclose(fd) if(fd>=0){ while(0 != ::close(fd)) { if(errno != EINTR) break; }; fd=-1; }
