@@ -8,23 +8,27 @@
 #include "lockable.h"
 #include <header.h>
 
-// Life line (process states) of a file description item
-#define FIST_FRESH 		0
-#define FIST_INITED		1
-#define FIST_DLPENDING		4
-#define FIST_DLGOTHEAD		8
-#define FIST_DLRECEIVING	16
-#define FIST_COMPLETE		32
-// usually unreachable states
-#define FIST_LOCALERROR 	999
-#define FIST_ERRNOUSER		9999
+class fileitem;
+typedef SHARED_PTR<fileitem> tFileItemPtr;
+typedef MYSTD::multimap<mstring, tFileItemPtr> tFiGlobMap;
 
 //! Base class containing all required data and methods for communication with the download sources
-class fileitem_base : public condition
+class fileitem : public condition
 {
-	friend class fileitem;
 public:
-	virtual ~fileitem_base();
+
+	// Life cycle (process states) of a file description item
+	enum FiStatus
+	{
+
+	FIST_FRESH, FIST_INITED, FIST_DLPENDING, FIST_DLGOTHEAD, FIST_DLRECEIVING,
+	FIST_COMPLETE,
+	// error cases: downloader reports its error or last user told downloader to stop
+	FIST_DLERROR,
+	FIST_DLSTOP // assumed to not have any users left
+	};
+
+	virtual ~fileitem();
 	
 	// initialize file item, return the status
 	virtual FiStatus Setup(bool bDynType);
@@ -35,9 +39,9 @@ public:
 	virtual ssize_t SendData(int confd, int filefd, off_t &nSendPos, size_t nMax2SendNow)=0;
 	
 	// downloader instruments
-	typedef extended_bool<bool, false> SuccessWithTransErrorFlag;
-	virtual SuccessWithTransErrorFlag DownloadStartedStoreHeader(const header & h, const char *pNextData,
-			bool bForcedRestart) =0;
+	//typedef extended_bool<bool, false> SuccessWithTransErrorFlag;
+	virtual bool DownloadStartedStoreHeader(const header & h, const char *pNextData,
+			bool bForcedRestart, bool &bDoCleanRestart) =0;
 	void IncDlRefCount();
 	void DecDlRefCount(const mstring & sReasonStatusLine);
 	//virtual void SetFailureMode(const mstring & message, FiStatus fist=FIST_ERROR,
@@ -68,85 +72,90 @@ public:
 
 	uint64_t m_nIncommingCount;
 	off_t m_nSizeSeen;
+	off_t m_nRangeLimit;
 	
 	bool m_bCheckFreshness;
-
 	// those is only good for very special purposes [tm]
 	bool m_bHeadOnly;
-	off_t m_nRangeLimit;
 
 protected:
-
 	bool m_bAllowStoreData;
-
-	fileitem_base(mstring);
-	fileitem_base();
+	fileitem();
 	off_t m_nSizeChecked;
 	header m_head;
 	int m_filefd;
 	int m_nDlRefsCount;
 	FiStatus m_status;
-
-	time_t m_nTimeExpireAt; //! when to consider this item obsolete
-	static time_t m_nEarliestExpiration; //! when to consider a closer look for expired items
-
-	mstring m_sPathAbs, m_sPathRel;
-
-	time_t m_nTimeDlDone;
-
-public:
-	virtual void Unreg() {}; // to decrease dependent user count when used in global contents
-
+	mstring m_sPathRel;
+	time_t m_nTimeDlStarted, m_nTimeDlDone;
 
 private:
-	void Init();
+	// helper data for global registration control. Access is synchronized by the global lock,
+	// not the internal lock here
+	tFiGlobMap::iterator m_globRef;
+	int usercount;
+	friend class fileItemMgmt;
 };
 
-typedef SHARED_PTR<fileitem_base> tFileItemPtr;
-
-#ifndef MINIBUILD
-
 // dl item implementation with storage on disk
-class fileitem_with_storage : public fileitem_base
+class fileitem_with_storage : public fileitem
 {
 public:
-	inline fileitem_with_storage(cmstring &s) : fileitem_base(s) {};
+	inline fileitem_with_storage(cmstring &s) {m_sPathRel=s;};
 	virtual ~fileitem_with_storage();
 	// send helper like wrapper for sendfile. Just declare virtual here to make it better customizable later.
 	virtual ssize_t SendData(int confd, int filefd, off_t &nSendPos, size_t nMax2SendNow);
-	virtual SuccessWithTransErrorFlag DownloadStartedStoreHeader(const header & h, const char *pNextData,
-			bool bForcedRestart);
+	virtual bool DownloadStartedStoreHeader(const header & h, const char *pNextData,
+			bool bForcedRestart, bool&);
 	virtual bool StoreFileData(const char *data, unsigned int size);
+
+	inline static mstring NormalizePath(cmstring &sPathRaw)
+	{
+		return acfg::stupidfs ? DosEscape(sPathRaw) : sPathRaw;
+	}
 };
 
-typedef MYSTD::pair<int,tFileItemPtr> tFileRefEntry;
-typedef MYSTD::multimap<mstring, tFileRefEntry> tFiGlobMap;
+#ifndef MINIBUILD
 
-//! This is a fileitem class which is aware of other items and download sources,
-//! includes global download state management
-class fileitem : public fileitem_with_storage
+// auto_ptr like object that "owns" a fileitem while it's registered in the global
+// access table. Might cause item's deletion when destroyed
+class fileItemMgmt
 {
 public:
-	virtual void Unreg(); // decrease dependent user count
-
 	// related to GetRegisteredFileItem but used for registration of custom file item
 	// implementations created elsewhere (which still need to obey regular work flow)
-	static bool RegisterFileItem(SHARED_PTR<fileitem> spCustomFileItem);
+	static fileItemMgmt RegisterFileItem(tFileItemPtr spCustomFileItem);
 
 	// public constructor wrapper, get a unique object from the map or a new one
-	static tFileItemPtr GetRegisteredFileItem(cmstring &sPath, bool bConsiderAltStore);
+	static fileItemMgmt GetRegisteredFileItem(cmstring &sPathUnescaped, bool bConsiderAltStore);
 
 	//! @return: true iff there is still something in the pool for later cleaning
-	static time_t DoDelayedUnregAndCheck();
+	static time_t BackgroundCleanup();
 
 	static void dump_status();
 
-protected:
-	fileitem(cmstring &s);
-	tFiGlobMap::iterator m_globRef;
+	// when copied around, invalidates the original reference
+	~fileItemMgmt();
+	inline fileItemMgmt() {}
+	fileItemMgmt(const fileItemMgmt &src);
+	fileItemMgmt& operator=(const fileItemMgmt &src);
+
+	inline fileitem* operator->() const {return m_ptr.get();}
+	inline tFileItemPtr & get() {return m_ptr;}
+	inline operator bool() const {return (bool) m_ptr;}
+
+	// deletes global registration and replaces m_ptr with another copy
+	void ReplaceWithLocal(fileitem* replacement);
+
+private:
+	tFileItemPtr m_ptr;
+	void Unreg();
 };
-#endif
+#else
+#define fileItemMgmt void
+#endif // not MINIBUILD
 
 #endif
 
 
+
