@@ -6,9 +6,12 @@
 #include "header.h"
 #include "acbuf.h"
 #include <sys/types.h>
+#include "fileitem.h"
 
+/*
 class fileitem_base;
 typedef SHARED_PTR<fileitem_base> tFileItemPtr;
+*/
 
 class con;
 
@@ -66,18 +69,18 @@ class job {
       
       header *m_pReqHead; // copy of users requests header
 
-      tFileItemPtr m_pItem;
+      fileItemMgmt m_pItem;
       off_t m_nSendPos, m_nCurrentRangeLast;
       off_t m_nAllDataCount;
       
       unsigned long m_nChunkRemainingBytes;
-      USHORT m_type; // file type, bit mask
+      rechecks::eMatchType m_type;
       
       job(const job&);
       job & operator=(const job&);
 
-      const char * BuildAndEnqueHeader(const FiStatus &fistate, const off_t &nGooddataSize, header& respHead);
-      FiStatus _SwitchToPtItem(const mstring &fileLoc);
+      const char * BuildAndEnqueHeader(const fileitem::FiStatus &fistate, const off_t &nGooddataSize, header& respHead);
+      fileitem::FiStatus _SwitchToPtItem(const mstring &fileLoc);
       void SetErrorResponse(const char * errorLine, const char *szLocation=NULL);
       void HandleLocalDownload(const mstring &visPath,
     			const mstring &fsBase, const mstring &fsSubpath);
