@@ -7,8 +7,8 @@
 #include "acbuf.h"
 
 #ifdef DEBUG
-#define MTLOGDEBUG(x) { SendFmt()<< x << "<br>\n"; }
-#define MTLOGASSERT(x, y) {if(!(x)) SendFmt()<< "<font color=red><b>" << y << "</b></font><br>\n";}
+#define MTLOGDEBUG(x) { SendFmt << x << "\n<br>\n"; }
+#define MTLOGASSERT(x, y) {if(!(x)) SendFmt << "<div class=\"ERROR\">" << y << "</div>\n<br>\n";}
 //#define MTLOGVERIFY(x, y) MTLOGASSERT(x, y)
 #else
 #define MTLOGASSERT(x, y) {}
@@ -16,6 +16,8 @@
 //#define MTLOGVERIFY(x, y) x
 #endif
 
+#define MAINT_PFX "maint_"
+
 class tWUIPage
 {
 public:
@@ -30,9 +32,11 @@ public:
 protected:
 	inline void SendChunk(const mstring &x) { SendChunk(x.data(), x.size()); }
 
-	inline void SendChunk(const tSS &x){ SendChunk(x.data(), x.length()); }
-	virtual void SendChunk(const char *data, size_t size);
-	inline void SendChunk(const char *x) { SendChunk(x, x?strlen(x):0); }
+	inline void SendChunk(const tSS &x, bool b=false){ SendChunk(x.data(), x.length(), b); }
+	void SendChunk(const char *data, size_t size, bool bRemoteOnly=false);
+	inline void SendChunk(const char *x, bool b=false) { SendChunk(x, x?strlen(x):0, b); }
+	// for customization in base classes
+	virtual void AfterSendChunk(const char *data, size_t size) {};
 
 	bool SendRawData(const char *data, size_t len, int flags);
 	virtual void EndTransfer();
@@ -47,37 +51,31 @@ private:
 	mstring m_sHostname;
 
 public:
-
-	// dirty little helper to create an object which shares memory with its previous/future
-	// instances. Some methods are inlined in order to skip unnecessary dtor calls.
-	class tFmtSendObj: public tSS
+	// dirty little RAII helper to send data after formating it, uses a shared buffer presented
+	// to the user via macro
+	class tFmtSendObj
 	{
 	public:
-		inline tFmtSendObj(tWUIPage &p) : m_parent(p)
-		{
-			m_buf = m_parent.m_fmtBuf;
-			m_nCapacity = m_parent.m_fmtSize;
-		}
+		inline tFmtSendObj(tWUIPage *p, bool remoteOnly)
+		: m_parent(*p), m_bRemoteOnly(remoteOnly) { }
 		inline ~tFmtSendObj()
 		{
-			if (!empty())
+			if (!m_parent.m_fmtHelper.empty())
 			{
-				m_parent.SendChunk(data(), length());
-				m_parent.m_fmtBuf = m_buf;
-				m_parent.m_fmtSize = m_nCapacity;
-				m_buf = NULL;
-				m_nCapacity = 0;
+				m_parent.SendChunk(m_parent.m_fmtHelper, m_bRemoteOnly);
+				m_parent.m_fmtHelper.clear();
 			}
 		}
-	private:
 		tWUIPage &m_parent;
+	private:
 		tFmtSendObj operator=(const tWUIPage::tFmtSendObj&);
+		bool m_bRemoteOnly;
 	};
 
-#define SendFmt() tFmtSendObj(*this)
-	char *m_fmtBuf;
-	size_t m_fmtSize;
+#define SendFmt tFmtSendObj(this, false).m_parent.m_fmtHelper
+#define SendFmtRemote tFmtSendObj(this, true).m_parent.m_fmtHelper
 
+	tSS m_fmtHelper;
 };
 
 void DispatchAndRunMaintTask(cmstring &cmd, int fd, const char *auth);
