@@ -162,7 +162,7 @@ bool ParseKeyValLine(const mstring & sIn, mstring & sOutKey, mstring & sOutVal);
 class tHttpUrl
 {
 public:
-	bool SetHttpUrl(cmstring &uri);
+	bool SetHttpUrl(cmstring &uri, bool unescape = true);
 	mstring ToURI(bool bEscaped) const;
 	mstring sHost, sPath, sUserPass;
 	bool bIsTransferlEncoded;
