@@ -10,7 +10,7 @@
 
 #include "cacheman.h"
 
-class pkgmirror: public tCacheMan
+class pkgmirror: public tCacheMan, ifileprocessor
 {
 public:
 	pkgmirror(int);
