@@ -9,7 +9,7 @@
 #include "csmapping.h"
 
 
-class pkgimport : public tCacheMan
+class pkgimport : public tCacheMan, ifileprocessor
 {
 
 public:
