@@ -24,7 +24,7 @@ private:
 class tDeleter : public tStaticFileSend
 {
 public:
-	inline tDeleter(int fd) : tStaticFileSend(fd, "delconfirm.html", "text/html", "200") {}
+	inline tDeleter(int fd) : tStaticFileSend(fd, "delconfirm.html", "text/html", "200 OK") {}
 	virtual void ModContents(mstring & contents, cmstring &cmd);
 };
 
