@@ -12,7 +12,7 @@
 #include <sys/socket.h>
 #include <sys/un.h>
 #include <netdb.h>
-
+#include <unistd.h>
 #include <cstddef>
 
 using namespace MYSTD;
@@ -34,14 +34,26 @@ using namespace MYSTD;
 #endif
 
 void termsocket(int);
+inline void termsocket_quick(int fd)
+{
+	if(fd<0)
+		return;
+	::shutdown(fd, SHUT_RDWR);
+	while(0 != ::close(fd))
+	{
+		if(errno != EINTR) break;
+	};
+	fd=-1;
+}
 
-
-#define forceShutdownClose(fd) if(fd>=0) { ::shutdown(fd, O_RDWR); while(0 != ::close(fd)) { if(errno != EINTR) break; }; fd=-1; }
-// DUDES, who writes TCP implementations... why can this just not be done easy and reliable? Why do we need hacks like termsocket?
-// For details, see: http://blog.netherlabs.nl/articles/2009/01/18/the-ultimate-so_linger-page-or-why-is-my-tcp-not-reliable
-#define flushShutdownClose(fd) if(fd>=0) { ::termsocket(fd); fd=-1; }
-
-
+inline bool check_read_state(int fd)
+{
+	fd_set rfds;
+	FD_ZERO(&rfds);
+	FD_SET(fd, &rfds);
+	struct timeval tv = { 0, 0};
+	return (1 == select(fd + 1, &rfds, NULL, NULL, &tv) && FD_ISSET(fd, &rfds));
+}
 
 
 #endif /*SOCKIO_H_*/
