@@ -15,6 +15,7 @@
 #include <string>
 #include <deque>
 #include <map>
+#include <list>
 #include <algorithm>
 
 using namespace MYSTD;
@@ -301,13 +302,14 @@ inline void AddRemapFlag(const string & token, const string &repname)
 	if(!ParseOptionLine(token, key, value, true))
 		return;
 
-	if(value.empty())
-		return;
-
 	tRepoData &where = repoparms[repname];
 
 	if(key=="keyfile")
 	{
+
+		if(value.empty())
+			return;
+
 		//if(!startsWithSz(value, "/"))
 		//	value.insert(0, "/");
 		if (acfg::debug&LOG_FLUSH)
@@ -317,12 +319,31 @@ inline void AddRemapFlag(const string & token, const string &repname)
 	}
 	else if(key=="deltasrc")
 	{
+
+		if(value.empty())
+			return;
+
 		if(!endsWithSzAr(value, "/"))
 			value+="/";
 
 		if(!where.m_deltasrc.SetHttpUrl(value))
 			cerr << "Couldn't parse Debdelta source URL, ignored " <<value <<endl;
 	}
+	else if(key=="proxy")
+	{
+		static MYSTD::list<tHttpUrl> alt_proxies;
+		tHttpUrl cand;
+		if(value.empty() || cand.SetHttpUrl(value))
+		{
+			alt_proxies.push_back(cand);
+			where.m_pProxy = & alt_proxies.back();
+		}
+		else
+		{
+			cerr << "Warning, failed to parse proxy setting " << value << " , "
+					<< endl << "ignoring it" <<endl;
+		}
+	}
 }
 
 tStrDeq ExpandFileTokens(cmstring &token)
@@ -970,7 +991,7 @@ void ReadConfigDirectory(const char *szPath, bool bTestMode)
 #endif
 }
 
-void PostProcConfig() 
+void PostProcConfig(bool bDumpConfig)
 {
 	
 	if(port.empty()) // heh?
@@ -1094,11 +1115,12 @@ void PostProcConfig()
 	   pipelinelen = 1;
    }
 
-   if (acfg::debug >= LOG_MORE)
+   if (acfg::debug >= LOG_MORE || bDumpConfig)
 	{
+	   MYSTD::ostream &cmine(bDumpConfig ? cout : cerr);
 		for (UINT i = 0; i < _countof(n2sTbl); i++)
 			if (n2sTbl[i].ptr)
-				cerr << n2sTbl[i].name << " = " << *(n2sTbl[i].ptr) << endl;
+				cmine << n2sTbl[i].name << " = " << *(n2sTbl[i].ptr) << endl;
 
 		if (acfg::debug >= LOG_DEBUG)
 		{
@@ -1109,16 +1131,16 @@ void PostProcConfig()
 					cerr << n2sTbl[i].name << " = ";
 					for (const char *p = n2sTbl[i].ptr->c_str(); *p; p++)
 						if ('\\' == *p)
-							cerr << "\\\\";
+							cmine << "\\\\";
 						else
-							cerr << *p;
-					cerr << endl;
+							cmine << *p;
+					cmine << endl;
 				}
 		}
 
 		for (UINT i = 0; i < _countof(n2iTbl); i++)
 			if (n2iTbl[i].ptr)
-				cerr << n2iTbl[i].name << " = \"" << *(n2iTbl[i].ptr) << "\"\n";
+				cmine << n2iTbl[i].name << " = \"" << *(n2iTbl[i].ptr) << "\"\n";
 	}
 
 #ifndef DEBUG
@@ -1137,7 +1159,7 @@ void PostProcConfig()
 	   p->cmdRel = "logger wanna/disconnect";
 	   it->second.m_pHooks = p;
    }
-#endif
+
 
    if(debug == -42)
    {
@@ -1160,6 +1182,7 @@ void PostProcConfig()
 	   exit(1);
 	   */
    }
+#endif
 } // PostProcConfig
 
 //! @brief Fires hook callbacks in the background thread
