@@ -88,7 +88,7 @@ int debug(3), foreground(true);
 int debug(0), foreground(false);
 #endif
 
-tHttpUrl proxy_info; // abuses path member for auth data
+tHttpUrl proxy_info;
 
 string cacheDirSlash; // guaranteed to have a trailing path separator
 
