@@ -79,7 +79,13 @@ void err(const char *msg, const char *client)
 	lockguard g(&mx);
 
 	if(!fErr.is_open())
+	{
+/*#ifdef DEBUG
+		cerr << msg <<endl;
+#endif
+*/
 		return;
+	}
 	
 	static char buf[32];
 	const time_t tm=time(NULL);
@@ -319,6 +325,10 @@ string GetStatReport()
 #endif
 
 
+}
+
+
+
 errnoFmter::errnoFmter(const char *prefix)
 {
 	char buf[32];
@@ -339,8 +349,6 @@ errnoFmter::errnoFmter(const char *prefix)
 #endif
 }
 
-}
-
 #ifdef DEBUG
 
 static lockable mx_dbgStackMark;
