@@ -206,6 +206,7 @@ int main(int argc, char **argv)
     }
     
     LPCSTR PRINTCFGVAR=getenv("PRINTCFGVAR");
+    bool bDumpCfg(false);
 
     for (char **p=argv+1; p<argv+argc; p++)
      {
@@ -217,6 +218,10 @@ int main(int argc, char **argv)
         	else
         		usage();
 		}
+        else if(!strcmp(*p, "-p"))
+        {
+        	bDumpCfg=true;
+        }
 		else if(**p) // not empty
 		{
 			if(!acfg::SetOption(*p, false))
@@ -237,8 +242,11 @@ int main(int argc, char **argv)
         exit(1);
     }
 
-	acfg::PostProcConfig();
-    
+	acfg::PostProcConfig(bDumpCfg);
+
+	if(bDumpCfg)
+		exit(EXIT_SUCCESS);
+
     SetupCacheDir();
 
 	extern mstring sReplDir;
@@ -283,7 +291,9 @@ static void usage() {
 		"Options:\n"
 		"-h: this help message\n"
 		"-c: configuration directory\n"
-		"-e: on startup, run expiration once\n\n"
+		"-e: on startup, run expiration once\n"
+		"-p: print configuration and exit\n"
+		"\n"
 		"Most interesting variables:\n"
 		"ForeGround: Don't detach (default: 0)\n"
 		"Port: TCP port number (default: 3142)\n"
