@@ -44,7 +44,7 @@ tWuiBgTask::~tWuiBgTask()
 }
 
 // the obligatory definition of static members :-(
-weak_ptr<tWuiBgTask::tProgressTracker> tWuiBgTask::g_pTracker;
+SMARTPTR_SPACE::weak_ptr<tWuiBgTask::tProgressTracker> tWuiBgTask::g_pTracker;
 
 void _AddFooter(tSS &msg)
 {
