@@ -101,91 +101,6 @@ tCacheMan::tIfileAttribs &tCacheMan::SetFlags(cmstring &sPathRel)
 	return sPathRel.empty() ? attr_dummy : m_indexFilesRel[sPathRel];
 }
 
-bool tCacheMan::FindURL_InitDload(tFileItemPtr fi, cmstring &sFilePathRel,
-		mstring &sErr, tHttpUrl *retURL)
-{
-	StartDlder();
-	if(!m_pDlcon)
-	{
-		sErr="General error while creating remote connection";
-		return false;
-	}
-
-	/*
-	 * Three ways to find the source to download from:
-	 * 1. Interpret the base directory as repository name
-	 * 2. Interpret the whole subpath as URL (host/path)
-	 * 3. Use X-Original-Source (which might be broken)
-	 */
-
-	// abusing url class to extract base directory
-
-	tHttpUrl url;
-	const acfg::tRepoData *pBackends(NULL);
-
-	if (!url.SetHttpUrl(sFilePathRel))
-	{
-		// should never happen, though
-		sErr=sFilePathRel+" does not seem to contain valid repository or host name.";
-		return false;
-	}
-
-	string sOrig;
-	{
-		lockguard g(fi.get());
-		const header & pHead=fi->GetHeaderUnlocked();
-		if (pHead.h[header::XORIG])
-			sOrig=pHead.h[header::XORIG];
-	}
-
-	pBackends = acfg::GetBackendVec(url.sHost);
-
-	if (pBackends)
-		// HIT, control by backend scheme, strip leading slash from path
-		 m_pDlcon->AddJob(fi, pBackends, url.sPath.substr(1));
-
-	if(!pBackends || retURL)
-	{
-		tHttpUrl urlOrig;
-		if(startsWithSz(sOrig, "http://") && urlOrig.SetHttpUrl(sOrig))
-		{
-			// ok, looks valid, is it better than the one from the path?
-			if(url != urlOrig)
-			{
-				if(urlOrig.sHost.find(".") != stmiss)
-				{
-					if(url.sHost.find(".") == stmiss)
-					{
-						// dir has no dots, x-orig-url host has -> prefer x-orig url
-						url=urlOrig;
-					}
-				}
-				else // no dots in urlOrig host, most likely broken
-				{
-					/*
-					 * if the directory has dots, use it and be quiet (unless verbosity is enabled).
-					 * Otherwise, warn the user.
-					 * */
-					if (m_bVerbose || url.sHost.find(".") != stmiss)
-					{
-						SendFmt <<"<span class=\"WARNING\">Code 520824! "
-							"Read the manual about known bugs! Attempting to use "
-								<< url.sHost << " as hostname</span>";
-					}
-				}
-			}
-		}
-	}
-
-	if(!pBackends)
-		m_pDlcon->AddJob(fi, url);
-
-	if(retURL)
-		*retURL = url;
-
-	return true;
-}
-
 bool tCacheMan::IsDeprecatedArchFile(cmstring &sFilePathRel)
 {
 	tStrPos pos = sFilePathRel.rfind("/dists/");
@@ -247,6 +162,9 @@ bool tCacheMan::IsDeprecatedArchFile(cmstring &sFilePathRel)
 bool tCacheMan::Download(const MYSTD::string & sFilePathRel, bool bIndexFile,
 		int nVerbosity, tGetItemDelegate *pItemDelg, const char *pForcedURL)
 {
+
+	LOGSTART("tCacheMan::Download");
+
 	mstring sErr;
 	bool bSuccess=false;
 #ifdef DEBUG_FLAGS
@@ -264,11 +182,12 @@ bool tCacheMan::Download(const MYSTD::string & sFilePathRel, bool bIndexFile,
 
 #define GOTOREPMSG(x) {sErr = x; bSuccess=false; goto rep_dlresult; }
 
-	tHttpUrl origURL;
+	tHttpUrl url;
 
-	// handle alternative behavior in the first or last cycles
-	if (pForcedURL && !origURL.SetHttpUrl(pForcedURL))
-		return false;
+	const acfg::tRepoData *pBackends(NULL);
+	mstring sPatSuffix;
+
+	//bool bFallbackMethodUsed = false;
 
 	fileItemMgmt fi(pItemDelg
 			? pItemDelg->GetFileItemPtr()
@@ -309,13 +228,68 @@ bool tCacheMan::Download(const MYSTD::string & sFilePathRel, bool bIndexFile,
 			<< sFilePathRel	<< "...\n";
 	}
 
-	if (!FindURL_InitDload(fi.get(), sFilePathRel, sErr, &origURL))
-		GOTOREPMSG("Failed to resolve source URL");
+	if (!m_pDlcon)
+		m_pDlcon = new dlcon(true);
 
-	if (origURL.sHost.empty())
-		GOTOREPMSG("Failed to resolve original URL")
+	if (!m_pDlcon)
+		GOTOREPMSG("Item busy, cannot reload");
 
-	m_pDlcon->AddJob(fi.get(), origURL);
+	if (pForcedURL)
+	{
+		// handle alternative behavior in the first or last cycles
+		if (!url.SetHttpUrl(pForcedURL))
+			GOTOREPMSG("Invalid source URL");
+	}
+	else
+	{
+		// must have the URL somewhere
+		lockguard(fi.get().get());
+		const header &hor=fi->GetHeaderUnlocked();
+		if(hor.h[header::XORIG] && url.SetHttpUrl(hor.h[header::XORIG]))
+		{
+			ldbg("got the URL from fi header");
+		}
+		else
+		{
+			// ok, maybe it's not the file we have on disk, depending on how this method
+			// was called? get the URL from there?
+			header h;
+			h.LoadFromFile(acfg::cacheDirSlash + sFilePathRel + ".head");
+			if (h.h[header::XORIG] && url.SetHttpUrl(h.h[header::XORIG]))
+			{
+				ldbg("got the URL from related .head file");
+			}
+			else
+			{
+				// no luck... ok, enforce the backend/subpath method and hope it's there
+				// accessed as http/port80
+				// bFallbackMethodUsed = true;
+				tHttpUrl parser;
+				if (parser.SetHttpUrl(sFilePathRel)
+						&& 0 != (pBackends = acfg::GetBackendVec(parser.sHost)))
+				{
+					ldbg("guessed from the repository path which could be mapped to a backend")
+					sPatSuffix = parser.sPath;
+				}
+				else
+				{
+					ldbg("guessed purely from repository path, hostname is " << parser.sHost)
+					url=parser;
+				}
+			}
+		}
+	}
+	if (!pBackends) // we still might want to remap it
+	{
+		cmstring * pBackendName = acfg::GetRepNameAndPathResidual(url, sPatSuffix);
+		if (pBackendName)
+			pBackends = acfg::GetBackendVec(*pBackendName);
+	}
+
+	if(pBackends)
+		m_pDlcon->AddJob(fi.get(), pBackends, sPatSuffix);
+	else
+		m_pDlcon->AddJob(fi.get(), url);
 
 	m_pDlcon->WorkLoop();
 	if (fi->WaitForFinish(NULL) == fileitem::FIST_COMPLETE
@@ -604,7 +578,7 @@ bool tCacheMan::PatchFile(const string &srcRel,
 		SendChunk("Patching...<br>");
 
 	tSS cmd;
-	cmd << "cd '" << CACHE_BASE << "_actmp' && red patch.base < combined.diff";
+	cmd << "cd '" << CACHE_BASE << "_actmp' && red --silent patch.base < combined.diff";
 	if (::system(cmd.c_str()))
 	{
 		MTLOGASSERT(false, "Command failed: " << cmd);
@@ -902,8 +876,8 @@ bool tCacheMan::Propagate(cmstring &donorRel, tContId2eqClass::iterator eqClassI
 
 void tCacheMan::StartDlder()
 {
-	if(!m_pDlcon)
-		m_pDlcon=new dlcon(true);
+	if (!m_pDlcon)
+		m_pDlcon = new dlcon(true);
 }
 
 void tCacheMan::UpdateIndexFiles()
@@ -1425,6 +1399,8 @@ bool tCacheMan::ParseAndProcessIndexFile(ifileprocessor &ret, const MYSTD::strin
 		enumIndexType idxType)
 {
 
+	LOGSTART("expiration::_ParseAndProcessIndexFile");
+
 #ifdef DEBUG_FLAGS
 	bool bNix=StrHas(sPath, "/i18n/");
 #endif
@@ -1453,14 +1429,13 @@ bool tCacheMan::ParseAndProcessIndexFile(ifileprocessor &ret, const MYSTD::strin
 	}
 
 
-	LOGSTART("expiration::_ParseAndProcessIndexFile");
 	filereader reader;
 
 	if (!reader.OpenFile(CACHE_BASE+sPath))
 	{
 		if(! GetFlags(sPath).forgiveDlErrors) // that would be ok (added by ignorelist), don't bother
 		{
-			aclog::errnoFmter err;
+			errnoFmter err;
 			SendFmt<<"<span class=\"WARNING\">WARNING: unable to open "<<sPath
 					<<"(" << err << ")</span>\n<br>\n";
 		}
@@ -1789,7 +1764,7 @@ bool tCacheMan::ParseAndProcessIndexFile(ifileprocessor &ret, const MYSTD::strin
 	return reader.CheckGoodState(false);
 }
 
-void tCacheMan::_ProcessSeenIndexFiles()
+void tCacheMan::ProcessSeenIndexFiles(ifileprocessor &pkgHandler)
 {
 	LOGSTART("expiration::_ParseVolatileFilesAndHandleEntries");
 	for(tS2IDX::const_iterator it=m_indexFilesRel.begin(); it!=m_indexFilesRel.end(); it++)
@@ -1824,9 +1799,11 @@ void tCacheMan::_ProcessSeenIndexFiles()
 			continue;
 		}
 
+		//bool bNix=(it->first.find("experimental/non-free/binary-amd64/Packages.xz") != stmiss);
+
 		SendChunk(string("Parsing metadata in ")+it->first+"<br>\n");
 
-		if( ! ParseAndProcessIndexFile(*this, it->first, itype))
+		if( ! ParseAndProcessIndexFile(pkgHandler, it->first, itype))
 		{
 			if(!m_indexFilesRel[it->first].forgiveDlErrors)
 			{
@@ -1908,3 +1885,34 @@ void tCacheMan::PrintStats(cmstring &title)
 	}
 	SendFmt << "</table>";
 }
+
+#ifdef DEBUG
+int parseidx_demo(LPCSTR file)
+{
+
+	class tParser : public tCacheMan, ifileprocessor
+	{
+	public:
+		tParser() : tCacheMan(2) {};
+		inline int demo(LPCSTR file)
+		{
+			return !ParseAndProcessIndexFile(*this, file, GuessIndexTypeFromURL(file));
+		}
+		virtual void HandlePkgEntry(const tRemoteFileInfo &entry, bool bUncompressForChecksum)
+		{
+			cout << "Dir: " << entry.sDirectory << endl << "File: " << entry.sFileName << endl
+					<< "Checksum-" << entry.fpr.GetCsName() << ": " << entry.fpr.GetCsAsString()
+					<< endl << "ChecksumUncompressed: " << bUncompressForChecksum << endl <<endl;
+		}
+		virtual void UpdateFingerprint(const mstring &sPathRel, off_t nOverrideSize,
+				uint8_t *pOverrideSha1, uint8_t *pOverrideMd5) {};
+		virtual bool ProcessRegular(const mstring &sPath, const struct stat &) {return true;}
+		virtual bool ProcessOthers(const mstring &sPath, const struct stat &) {return true;}
+		virtual bool ProcessDirAfter(const mstring &sPath, const struct stat &) {return true;}
+		virtual void Action(const mstring &) {};
+	}
+	mgr;
+
+	return mgr.demo(file);
+}
+#endif
