@@ -22,10 +22,13 @@ bool CAddrInfo::Resolve(const string & sHostname, const string &sPort,
 {
 	LOGSTART2("CAddrInfo::Resolve", "Resolving " << sHostname);
 
+	LPCSTR port = sPort.empty() ? NULL : sPort.c_str();
+
 	static struct addrinfo hints =
 	{
 	// we provide numbers, no resolution needed; only supported addresses
-			AI_NUMERICSERV|AI_ADDRCONFIG, PF_UNSPEC, SOCK_STREAM, IPPROTO_TCP,
+			(port?AI_NUMERICSERV:0) | AI_ADDRCONFIG,
+			PF_UNSPEC, SOCK_STREAM, IPPROTO_TCP,
 			0, NULL, NULL, NULL };
 
 	// if only one family is specified, filter on this earlier
@@ -38,8 +41,7 @@ bool CAddrInfo::Resolve(const string & sHostname, const string &sPort,
 		m_resolvedInfo=NULL;
 	}
 
-	int r=getaddrinfo(sHostname.c_str(), sPort.c_str(), &hints,
-					&m_resolvedInfo);
+	int r=getaddrinfo(sHostname.c_str(), port, &hints, &m_resolvedInfo);
 
 	if (0!=r)
 	{
@@ -78,12 +80,14 @@ CAddrInfo::SPtr CAddrInfo::CachedResolve(const string & sHostname, const string
 {
 	//time_t timeExpired=time(NULL)+acfg::dnscachetime;
 	time_t now(time(0));
+	mstring dnsKey=sHostname+":"+sPort;
 
 	SPtr p;
 
 	{
 		lockguard g(&lockDnsCache);
-		SPtr & cand = mapDnsCache[sHostname];
+		SPtr localEntry;
+		SPtr & cand = acfg::dnscachetime>0 ? mapDnsCache[dnsKey] : localEntry;
 		if(cand && cand->m_nExpTime >= now)
 			return cand;
 		cand.reset(new CAddrInfo);
@@ -105,12 +109,13 @@ CAddrInfo::SPtr CAddrInfo::CachedResolve(const string & sHostname, const string
 	}
 	else // not good, remove from the cache again
 	{
-		aclog::err( (tSS()<<"Error resolving "<<sHostname<<": " <<sErrorMsgBuf).c_str());
+		aclog::err( (tSS()<<"Error resolving "<<dnsKey<<": " <<sErrorMsgBuf).c_str());
 		lockguard g(&lockDnsCache);
-		mapDnsCache.erase(sHostname);
+		mapDnsCache.erase(dnsKey);
 		p->unlock();
 		p.reset();
 	}
+
 	return p;
 }
 
