@@ -86,16 +86,18 @@ public:
 	{
 		string sErr;
 		tSS clientBufOut;
-		clientBufOut.init(32*1024); // out to be enough for any BTS response
+		clientBufOut.setsize(32*1024); // out to be enough for any BTS response
 
 		if(listBugsMode)
-			m_spOutCon = tcpconnect::CreateConnected("bugs.debian.org", "80", sErr);
+			m_spOutCon = tcpconnect::CreateConnected("bugs.debian.org", sDefPortHTTP,
+					sErr, 0,0,false,acfg::nettimeout);
 		else if(pConTgt)
 		{
 			tHttpUrl url;
 			if(!url.SetHttpUrl(*pConTgt))
 				return;
-			m_spOutCon = tcpconnect::CreateConnected(url.sHost , url.sPort, sErr);
+			m_spOutCon = tcpconnect::CreateConnected(url.sHost , url.GetPort(), sErr, 0,0,
+					false, acfg::nettimeout);
 
 			if(m_spOutCon)
 				clientBufOut << "HTTP/1.0 200 Connection established\r\n\r\n";
@@ -149,16 +151,14 @@ public:
 			if(FD_ISSET(ofd, &wfds))
 			{
 				// if the incoming request is from a proxy-style acng setup then we need to
-				// remove the HOST string from the header and replace it with something harmles.
+				// remove the HOST string from the prefix in the request header and replace it
+				// with something harmless.
 				// But if there is another web proxy, let it deal with the stuff...
-				if(!m_spOutCon->GetProxyData())
-				{
-					serverBufOut.move();
-					char *p = 0;
-					serverBufOut.flush();
-					while (0 != (p = strstr((char*)serverBufOut.rptr(), POSTMARK)))
-						memcpy(p, FAKEMARK, _countof(FAKEMARK) - 1);
-				}
+				serverBufOut.move();
+				char *p = 0;
+
+				while (0 != (p = strstr((char*) serverBufOut.rptr(), POSTMARK)))
+					memcpy(p, FAKEMARK, _countof(FAKEMARK) - 1);
 
 				if(serverBufOut.syswrite(ofd)<0)
 					return;
@@ -193,7 +193,7 @@ void con::WorkLoop() {
 	signal(SIGPIPE, SIG_IGN);
 	
     acbuf inBuf;
-    inBuf.init(32*1024);
+    inBuf.setsize(32*1024);
     
     int maxfd=m_confd;
     while(!m_bStopActivity) {
