@@ -14,11 +14,13 @@ using namespace MYSTD;
 // evil hack to simulate random disconnects
 //#define DISCO_FAILURE
 
+typedef MYSTD::pair<const tHttpUrl*,bool> tHostIsproxy;
+
 static const cmstring sGenericError("567 Unknown download error occured");
 
 dlcon::dlcon(bool bManualExecution, string *xff) :
 		m_bStopASAP(false), m_bManualMode(bManualExecution), m_nTempPipelineDisable(0),
-		m_proxied(!acfg::proxy_info.sHost.empty())
+		m_bProxyTot(false)
 {
 	LOGSTART("dlcon::dlcon");
 	m_wakepipe[0] = m_wakepipe[1] = -1;
@@ -35,6 +37,8 @@ struct tDlJob
 {
 	tFileItemPtr m_pStorage;
 	mstring sErrorMsg;
+	dlcon &m_parent;
+
 	inline bool HasBrokenStorage()
 	{
 		return (!m_pStorage || m_pStorage->GetStatus() > fileitem::FIST_COMPLETE);
@@ -51,9 +55,9 @@ struct tDlJob
 #define HINT_KILL_LAST_FILE 128
 #define HINT_TGTCHANGE 256
 
-	inline const tHttpUrl & GetPeerHost()
+	inline const tHttpUrl *GetPeerHost()
 	{
-		return m_pCurBackend ? *m_pCurBackend : m_fileUri;
+		return m_pCurBackend ? m_pCurBackend : &m_fileUri;
 	}
 
 	inline acfg::tRepoData::IHookHandler * GetConnStateTracker()
@@ -92,7 +96,7 @@ struct tDlJob
 	}
 
 	inline tDlJob(dlcon *p, tFileItemPtr pFi, const tHttpUrl& uri, int redirmax) :
-			m_pStorage(pFi), m_pBEdata(NULL),
+			m_pStorage(pFi), m_parent(*p), m_pBEdata(NULL),
 			m_fileUri(uri),
 			m_nRedirRemaining(redirmax)
 	{
@@ -101,7 +105,7 @@ struct tDlJob
 
 	inline tDlJob(dlcon *p, tFileItemPtr pFi, const acfg::tRepoData * pBackends,
 			const MYSTD::string & sPath, int redirmax) :
-			m_pStorage(pFi), m_pBEdata(pBackends),
+			m_pStorage(pFi), m_parent(*p), m_pBEdata(pBackends),
 			m_nRedirRemaining(redirmax)
 	{
 		Init();
@@ -134,7 +138,7 @@ struct tDlJob
 		mstring s;
 		for (const char *p = pNewUrl; *p; ++p)
 		{
-			if (isspace((unsigned char) *p))
+			if (isspace((unsigned char) *p) || *p == '%')
 			{
 				s += "%";
 				s += BytesToHexString((uint8_t*) p, 1);
@@ -155,29 +159,40 @@ struct tDlJob
 
 		return true;
 	}
-//
-//acfg::tRepoData::IHookHandler * tDlJob::GetConnStateObserver()
-//{
-//	return m_pBEdata ? m_pBEdata->m_pHooks : NULL;
-//}
 
-//
-//void tDlJob::BlacklistBackend()
-//{
-//	m_parent->m_MirrorHostBlacklist.insert(m_pCurBackend
-//			? m_pCurBackend->sHost
-//		    : m_fileUri.sHost);
-//}
+	inline tHostIsproxy GetConnectHost()
+	{
+		if (!m_parent.m_bProxyTot)
+		{
+			// otherwise consider using proxy
 
-// needs connectedHost, blacklist, output buffer from the parent, proxy mode?
-	inline void AppendRequest(tSS &head, cmstring &xff, bool proxied)
+			if (m_pBEdata && m_pBEdata->m_pProxy)
+			{
+				// do what the specific entry says
+				if(m_pBEdata->m_pProxy->sHost.empty())
+					return make_pair(GetPeerHost(), false);
+				return make_pair(m_pBEdata->m_pProxy, true);
+			}
+			if (!acfg::proxy_info.sHost.empty())
+				return make_pair(& acfg::proxy_info, true);
+			// ok, no proxy...
+		}
+		return make_pair(GetPeerHost(), false);
+	}
+
+	// needs connectedHost, blacklist, output buffer from the parent, proxy mode?
+	inline void AppendRequest(tSS &head, cmstring &xff)
 	{
 		LOGSTART("tDlJob::AppendRequest");
 
 		head << (m_pStorage->m_bHeadOnly ? "HEAD " : "GET ");
 
-		if (proxied)
+		tHostIsproxy conHostInfo = GetConnectHost();
+
+		if (conHostInfo.second)
+		{
 			head << RemoteUri(true);
+		}
 		else // only absolute path without scheme
 		{
 			if (m_pCurBackend) // base dir from backend definition
@@ -191,15 +206,15 @@ struct tDlJob
 
 		ldbg(RemoteUri(true));
 
-		head << " HTTP/1.1\r\n" << acfg::agentheader << "Host: " << GetPeerHost().sHost << "\r\n";
+		head << " HTTP/1.1\r\n" << acfg::agentheader << "Host: " << GetPeerHost()->sHost << "\r\n";
 
-		if (proxied) // proxy stuff, and add authorization if there is any
+		if (conHostInfo.second) // proxy stuff, and add authorization if there is any
 		{
 			ldbg("using proxy");
-			if(!acfg::proxy_info.sUserPass.empty())
+			if(!conHostInfo.first->sUserPass.empty())
 			{
 				head << "Proxy-Authorization: Basic "
-						<< EncodeBase64Auth(acfg::proxy_info.sUserPass) << "\r\n";
+						<< EncodeBase64Auth(conHostInfo.first->sUserPass) << "\r\n";
 			}
 			// Proxy-Connection is a non-sensical copy of Connection but some proxy
 			// might listen only to this one so better add it
@@ -207,11 +222,11 @@ struct tDlJob
 									: "Proxy-Connection: close\r\n");
 		}
 
-		const tHttpUrl *pCredSrc=m_pCurBackend ? m_pCurBackend : &m_fileUri;
-		if(pCredSrc && ! pCredSrc->sUserPass.empty())
+		const tHttpUrl& pSourceHost(* GetPeerHost());
+		if(!pSourceHost.sUserPass.empty())
 		{
 			head << "Authorization: Basic "
-				<< EncodeBase64Auth(pCredSrc->sUserPass) << "\r\n";
+				<< EncodeBase64Auth(pSourceHost.sUserPass) << "\r\n";
 		}
 
 		// either by backend or by host in file uri, never both
@@ -572,8 +587,8 @@ private:
 inline void dlcon::BlacklistMirror(tDlJobPtr & job, cmstring &msg)
 {
 	LOGSTART2("dlcon::BlacklistMirror", "blacklisting " <<
-			job->GetPeerHost().ToURI(false));
-	m_blacklist[make_pair(job->GetPeerHost().sHost, job->GetPeerHost().GetPort())] = msg;
+			job->GetPeerHost()->ToURI(false));
+	m_blacklist[make_pair(job->GetPeerHost()->sHost, job->GetPeerHost()->GetPort())] = msg;
 }
 
 
@@ -613,7 +628,7 @@ inline bool dlcon::SetupJobConfig(tDlJobPtr &job, mstring *pReasonMsg)
 	}
 
 	// ok, look for the mirror data itself
-	bliter = m_blacklist.find(make_pair(job->GetPeerHost().sHost, job->GetPeerHost().GetPort()));
+	bliter = m_blacklist.find(make_pair(job->GetPeerHost()->sHost, job->GetPeerHost()->GetPort()));
 	if(bliter == m_blacklist.end())
 		return true;
 	else
@@ -1038,24 +1053,21 @@ void dlcon::WorkLoop()
 
         		bool bUsed=false;
 #ifdef HAVE_SSL
-#define doconnect(x, t) tcpconnect::CreateConnected(x.sHost, x.GetPort(), \
-				sErrorMsg, &bUsed, m_qNewjobs.front()->GetConnStateTracker(), x.bSSL, t)
+#define doconnect(x, t) tcpconnect::CreateConnected(x->sHost, x->GetPort(), \
+				sErrorMsg, &bUsed, m_qNewjobs.front()->GetConnStateTracker(), x->bSSL, t)
 #else
-#define doconnect(x, t) tcpconnect::CreateConnected(x.sHost, x.GetPort(), \
+#define doconnect(x, t) tcpconnect::CreateConnected(x->sHost, x->GetPort(), \
 				sErrorMsg, &bUsed, m_qNewjobs.front()->GetConnStateTracker(), false, t)
 
 #endif
-
-        		if(!m_proxied)
-        			con = doconnect(m_qNewjobs.front()->GetPeerHost(), acfg::nettimeout);
-        		else
-        			con = doconnect(acfg::proxy_info,
-        				acfg::optproxytimeout>0 ? acfg::optproxytimeout : acfg::nettimeout);
+        		tHostIsproxy conHost = m_qNewjobs.front()->GetConnectHost();
+        		con = doconnect( conHost.first, (conHost.second && acfg::optproxytimeout>0)
+        				? acfg::optproxytimeout : acfg::nettimeout);
 
         		if(!con && acfg::optproxytimeout>0)
         		{
-        			ldbg("optional proxy timed out, disable");
-        			m_proxied=false;
+        			ldbg("optional proxy broken, disable");
+        			m_bProxyTot=true;
         			con = doconnect(m_qNewjobs.front()->GetPeerHost(), acfg::nettimeout);
         		}
 
@@ -1104,21 +1116,20 @@ void dlcon::WorkLoop()
         			continue;
         		}
 
-        		// direct/non-proxy mode and the wanted target host changed?
-        		if(!m_proxied
-        				// needs to send them for the connected target host
-        				&& ( cjob->GetPeerHost().sHost != con->GetHostname()
-        				|| cjob->GetPeerHost().GetPort() != con->GetPort()) )
+				// needs to send them for the connected target host
+        		tHostIsproxy hostNew=cjob->GetConnectHost();
+        		if(hostNew.first->sHost != con->GetHostname() ||
+        				hostNew.first->GetPort() != con->GetPort())
         		{
-        			LOG("host mismatch," << cjob->GetPeerHost().sHost << ":" <<
-        					cjob->GetPeerHost().GetPort() <<
+        			LOG("host mismatch," << hostNew.first->sHost << ":" <<
+        					hostNew.first->GetPort() <<
         					" vs. " << con->GetHostname() << ":"<<con->GetPort() <<
         					" -- stop sending requesting for now");
         			bStopRequesting=true;
         			break;
         		}
 
-				cjob->AppendRequest(m_sendBuf, m_sXForwardedFor, m_proxied);
+				cjob->AppendRequest(m_sendBuf, m_sXForwardedFor);
 				LOG("request added to buffer");
 				inpipe.push_back(cjob);
 				m_qNewjobs.pop_front();
