@@ -475,7 +475,7 @@ void expiration::Action(const string & cmd)
 
 		mstring delURL(cmd);
 		StrSubst(delURL, "justShow", "justRemove");
-		SendFmtRemote << "<a href=\"/"<<delURL<<"\">Delete all listed files</a> "
+		SendFmtRemote << "<a href=\""<<delURL<<"\">Delete all listed files</a> "
 				"(no further confirmation)<br>\n";
 		return;
 	}
