@@ -15,15 +15,18 @@ int falloc_helper(int fd, off_t start, off_t len)
 {
    return fallocate(fd, FALLOC_FL_KEEP_SIZE, start, len);
 }
+#else
+int falloc_helper(int, off_t, off_t)
+{
+   return 0;
+}
 #endif
 
-
-
 // linking not possible? different filesystems?
 bool FileCopy_generic(cmstring &from, cmstring &to)
 {
 	acbuf buf;
-	buf.init(50000);
+	buf.setsize(50000);
 	int in(-1), out(-1);
 
 	in=::open(from.c_str(), O_RDONLY);
