@@ -566,6 +566,13 @@ bool fileitem_with_storage::DownloadStartedStoreHeader(const header & h, const c
 		// else... nothing to fix since the expectation==reality
 
 		falloc_helper(m_filefd, hint_start, hint_length);
+    /*
+     * double-check the docs, this is probably not relevant for writting
+#ifdef HAVE_FADVISE
+	// optional, experimental
+		posix_fadvise(m_filefd, hint_start, hint_length, POSIX_FADV_SEQUENTIAL);
+#endif
+*/
 		
 		ldbg("Storing header as "+sHeadPath);
 		int count=m_head.StoreToFile(sHeadPath);
