@@ -526,7 +526,7 @@ void job::PrepareDownload() {
 		if (0==sReqPath.compare(0, 11, "apt-cacher/"))
 		sReqPath.erase(11);
 		
-		if(!tUrl.SetHttpUrl(sReqPath))
+		if(!tUrl.SetHttpUrl(sReqPath, false))
 		{
 			m_sMaintCmd="/";
 			return;
