@@ -194,7 +194,7 @@ bool ParseKeyValLine(const string & sIn, string & sOutKey, string & sOutVal)
 }
 
 
-bool tHttpUrl::SetHttpUrl(cmstring &sUrlRaw)
+bool tHttpUrl::SetHttpUrl(cmstring &sUrlRaw, bool unescape)
 {
 	sPort.clear();
 	sHost.clear();
@@ -205,7 +205,7 @@ bool tHttpUrl::SetHttpUrl(cmstring &sUrlRaw)
 #endif
 	bIsTransferlEncoded=false;
 	
-	mstring url = UrlUnescape(sUrlRaw);
+	mstring url = unescape ? UrlUnescape(sUrlRaw) : sUrlRaw;
 
 	trimBack(url);
 	trimFront(url);
