@@ -112,7 +112,7 @@ void pkgmirror::Action(const string &cmd)
 
 	if(m_bUseDelta)
 	{
-		if(::system("dpkg -h"))
+		if(::system("dpkg --version"))
 		{
 			SendChunk("<b>dpkg not found, Debdelta support disabled</b><br>\n");
 			m_bUseDelta=false;
