@@ -241,7 +241,7 @@ void pkgimport::Action(const string &cmd)
 		return;
 	}
 	
-	_ProcessSeenIndexFiles();
+	ProcessSeenIndexFiles(*this);
 
 	{
 		lockguard g(&abortMx);
