@@ -27,15 +27,28 @@ using namespace MYSTD;
 volatile int nConCount(0), nDisconCount(0), nReuseCount(0);
 #endif
 
+#ifdef HAVE_SSL
+#include <openssl/bio.h>
+#include <openssl/ssl.h>
+#include <openssl/err.h>
+#endif
+
 tcpconnect::tcpconnect() :	m_conFd(-1), m_pConnStateObserver(NULL)
+#ifdef HAVE_SSL
+,m_bio(NULL), m_ctx(NULL), m_ssl(NULL)
+#endif
 {
-	m_proxy = acfg::proxy_info.sHost.empty() ? NULL : &acfg::proxy_info;
 }
 
 tcpconnect::~tcpconnect()
 {
 	LOGSTART("tcpconnect::~tcpconnect, terminating outgoing connection class");
 	Disconnect();
+#ifdef HAVE_SSL
+	if(m_ctx)
+		SSL_CTX_free(m_ctx);
+	m_ctx=0;
+#endif
 }
 
 
@@ -139,12 +152,12 @@ static int connect_timeout(int sockfd, const struct sockaddr *addr, socklen_t ad
 	return 0;
 }
 
-bool tcpconnect::Connect(const string & sHostname, const string & sPort, string & sErrorMsg)
+inline bool tcpconnect::Connect(string & sErrorMsg, int timeout)
 {
-	LOGSTART2("tcpconnect::_Connect", "hostname: " << sHostname);
-	CAddrInfo::SPtr dns = m_proxy ?
-			CAddrInfo::CachedResolve(m_proxy->sHost, m_proxy->sPort, sErrorMsg)
-			: CAddrInfo::CachedResolve(sHostname, sPort, sErrorMsg);
+	LOGSTART2("tcpconnect::_Connect", "hostname: " << m_sHostName);
+
+	CAddrInfo::SPtr dns = CAddrInfo::CachedResolve(m_sHostName, m_sPort, sErrorMsg);
+
 	if(!dns)
 	{
 		USRDBG(sErrorMsg);
@@ -161,7 +174,7 @@ bool tcpconnect::Connect(const string & sHostname, const string & sPort, string
 			if (acfg::conprotos[i] != PF_UNSPEC && acfg::conprotos[i] != pInfo->ai_family)
 				continue;
 
-			ldbg("Creating socket for " << sHostname);
+			ldbg("Creating socket for " << m_sHostName);
 
 			if (pInfo->ai_socktype != SOCK_STREAM || pInfo->ai_protocol != IPPROTO_TCP)
 				continue;
@@ -180,15 +193,17 @@ bool tcpconnect::Connect(const string & sHostname, const string & sPort, string
 			}
 #endif
 
-			if (::connect_timeout(m_conFd, pInfo->ai_addr, pInfo->ai_addrlen, acfg::nettimeout) < 0)
+			if (::connect_timeout(m_conFd, pInfo->ai_addr, pInfo->ai_addrlen, timeout) < 0)
+			{
+				if(errno==ETIMEDOUT)
+					sErrorMsg="Connection timeout";
 				continue;
+			}
 #ifdef DEBUG
 			__sync_fetch_and_add(&nConCount, 1);
 #endif
 			ldbg("connect() ok");
 			set_nb(m_conFd);
-			m_sHostName = sHostname;
-			m_sPort = sPort;
 			return true;
 		}
 	}
@@ -197,7 +212,7 @@ bool tcpconnect::Connect(const string & sHostname, const string & sPort, string
 	sErrorMsg = "500 Connection failure";
 #else
 	// format the last available error message for the user
-	sErrorMsg=aclog::errnoFmter("500 Connection failure: ");
+	sErrorMsg=errnoFmter("500 Connection failure: ");
 #endif
 	ldbg("Force reconnect, con. failure");
 	Disconnect();
@@ -213,6 +228,11 @@ void tcpconnect::Disconnect()
 		__sync_fetch_and_add(&nDisconCount, 1);
 #endif
 
+#ifdef HAVE_SSL
+	if(m_bio)
+		BIO_free_all(m_bio), m_bio=NULL;
+#endif
+
 	if(m_conFd>=0 && m_pConnStateObserver)
 		m_pConnStateObserver->JobRelease();
 
@@ -226,39 +246,86 @@ lockable conPoolMx;
 using namespace MYSTD;
 struct tHostHint // could derive from pair but prefer to save some bytes with references
 {
-	cmstring &pHost, &pPort;
-	inline tHostHint(cmstring &h, cmstring &p, time_t d=0) : pHost(h), pPort(p) {};
+	cmstring pHost, pPort;
+#ifdef HAVE_SSL
+	bool bSsled;
+	inline tHostHint(cmstring &h, cmstring &p, bool bSsl) : pHost(h), pPort(p), bSsled(bSsl) {};
 	bool operator<(const tHostHint &b) const
 	{
 		int rel=pHost.compare(b.pHost);
-		return rel ? rel < 0 : pPort < b.pPort;
+		if(rel)
+			return rel < 0;
+		if(pPort != b.pPort)
+			return pPort < b.pPort;
+		return bSsled < b.bSsled;
 	}
+#else
+	inline tHostHint(cmstring &h, cmstring &p) : pHost(h), pPort(p) {};
+		bool operator<(const tHostHint &b) const
+		{
+			int rel=pHost.compare(b.pHost);
+			if(rel)
+				return rel < 0;
+			return pPort < b.pPort;
+		}
+#endif
+
 };
 typedef multimap<tHostHint, MYSTD::pair<tTcpHandlePtr, time_t> > tConPool;
 tConPool spareConPool;
 
 tTcpHandlePtr tcpconnect::CreateConnected(cmstring &sHostname, cmstring &sPort,
-		mstring &sErrOut, bool *pbSecondHand, acfg::tRepoData::IHookHandler *pStateTracker)
+		mstring &sErrOut, bool *pbSecondHand, acfg::tRepoData::IHookHandler *pStateTracker
+		,bool bSsl, int timeout)
 {
-	LOGSTART2s("tcpconnect::CreateConnected", "hostname: " << sHostname);
+	LOGSTART2s("tcpconnect::CreateConnected", "hostname: " << sHostname << ", port: " << sPort
+			<< (bSsl?" with ssl":" , no ssl"));
 
 	tTcpHandlePtr p;
+#ifndef HAVE_SSL
+	if(bSsl)
+	{
+		aclog::err("E_NOTIMPLEMENTED: SSL");
+		return p;
+	}
+#endif
+
 	bool bReused=false;
-	tHostHint key(sHostname, sPort);
+	tHostHint key(sHostname, sPort
+#ifdef HAVE_SSL
+			, bSsl
+#endif
+	);
 
 #ifdef NOCONCACHE
 	p.reset(new tcpconnect);
 	if(!p || !p->Connect(sHostname, sPort, sErrOut) || p->GetFD()<0) // failed or worthless
 		p.reset();
 #else
-	{
+	{ // mutex context
 		lockguard __g(conPoolMx);
 		tConPool::iterator it=spareConPool.find(key);
 		if(spareConPool.end() == it)
 		{
 			p.reset(new tcpconnect);
-			if(!p || !p->Connect(sHostname, sPort, sErrOut) || p->GetFD()<0) // failed or worthless
+			if(p)
+			{
+				p->m_sHostName=sHostname;
+				p->m_sPort=sPort;
+			}
+
+			if(!p || !p->Connect(sErrOut, timeout) || p->GetFD()<0) // failed or worthless
 				p.reset();
+#ifdef HAVE_SSL
+			else if(bSsl)
+			{
+				if(!p->SSLinit(sErrOut, sHostname, sPort))
+				{
+					p.reset();
+					LOG("ssl init error");
+				}
+			}
+#endif
 		}
 		else
 		{
@@ -309,7 +376,7 @@ void tcpconnect::RecycleIdleConnection(tTcpHandlePtr & handle)
 	if(check_read_state(handle->GetFD()))
 	{
 		acbuf checker;
-		checker.init(300000);
+		checker.setsize(300000);
 		checker.sysread(handle->GetFD());
 
 	}
@@ -323,8 +390,11 @@ void tcpconnect::RecycleIdleConnection(tTcpHandlePtr & handle)
 	// a DOS?
 	if(spareConPool.size()<50)
 	{
-		spareConPool.insert(make_pair(tHostHint(handle->GetHostname(), handle->GetPort()),
-				make_pair(handle, now)));
+		spareConPool.insert(make_pair(tHostHint(handle->GetHostname(), handle->GetPort()
+#ifdef HAVE_SSL
+				,handle->m_bio
+#endif
+		),make_pair(handle, now)));
 
 #ifndef MINIBUILD
 		g_victor.ScheduleFor(now+TIME_SOCKET_EXPIRE_CLOSE, cleaner::TYPE_EXCONNS);
@@ -410,3 +480,78 @@ void tcpconnect::dump_status()
 
 	aclog::err(msg);
 }
+#ifdef HAVE_SSL
+bool tcpconnect::SSLinit(mstring &sErr, cmstring &sHostname, cmstring &sPort)
+{
+	SSL * ssl(NULL);
+	int hret(0);
+	LPCSTR perr(0);
+
+	// cleaned up in the destructor on EOL
+	if(!m_ctx)
+	{
+		m_ctx = SSL_CTX_new(SSLv23_client_method());
+		if (!m_ctx)
+			goto ssl_init_fail;
+
+		SSL_CTX_load_verify_locations(m_ctx, acfg::cafile.empty() ? NULL : acfg::cafile.c_str(),
+			acfg::capath.empty() ? NULL : acfg::capath.c_str());
+
+	}
+
+	ssl = SSL_new(m_ctx);
+	if(!ssl)
+		goto ssl_init_fail;
+
+	// mark it connected and prepare for non-blocking mode
+ 	SSL_set_connect_state(ssl);
+ 	SSL_set_mode(ssl, SSL_MODE_AUTO_RETRY
+ 			| SSL_MODE_ACCEPT_MOVING_WRITE_BUFFER
+ 			| SSL_MODE_ENABLE_PARTIAL_WRITE);
+
+ 	// make it blocking for sure otherwise SSL_connect would become messy
+	set_block(m_conFd);
+
+ 	if((hret=SSL_set_fd(ssl, m_conFd)) != 1)
+ 		goto ssl_init_fail_retcode;
+	if((hret=SSL_connect(ssl)) != 1)
+		goto ssl_init_fail_retcode;
+
+ 	m_bio = BIO_new(BIO_f_ssl());
+ 	if(!m_bio)
+ 	{
+ 		perr="IO initialization error";
+ 		goto ssl_init_fail;
+ 	}
+ 	// not sure we need it but maybe the handshake can access this data
+ 	BIO_set_conn_hostname(m_bio, sHostname.c_str());
+ 	BIO_set_conn_port(m_bio, sPort.c_str());
+
+ 	BIO_set_ssl(m_bio, ssl, BIO_NOCLOSE);
+
+ 	BIO_set_nbio(m_bio, 1);
+	set_nb(m_conFd);
+
+	hret=SSL_get_verify_result(ssl);
+	if( hret != X509_V_OK)
+	{
+		perr=X509_verify_cert_error_string(hret);
+		goto ssl_init_fail;
+	}
+
+	return true;
+
+	ssl_init_fail_retcode:
+
+	perr=ERR_reason_error_string(SSL_get_error(ssl, hret));
+
+	ssl_init_fail:
+
+	if(!perr)
+		perr=ERR_reason_error_string(ERR_get_error());
+	sErr="SSL error: ";
+	sErr+=(perr?perr:"Generic SSL failure");
+	return false;
+}
+
+#endif
